<?php

namespace Modules\Pp\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Pp\Entities\Order;
use Modules\Pp\Entities\OrderAttachment;
use Illuminate\Support\Facades\DB;


class OrderController extends Controller
{
    /**
     * save
     * This function is used to save new order to database.
     * @param request This is the paramter to save function
     * @return Json This returns as response.
     */
    public function save(Request $request)
    {

        try {

            $order = new Order();
            $order->customer_name = $request->get('name');
            $order->purchase_order = $request->get('txtPurchase');
           // $order->product_name = $request->get('txtProductName');
            $order->bill_address = $request->get('txtBillAddress');
            $order->delivery_address = $request->get('txtDeliveryAddress');
            $order->cosignee_details = $request->get('txtCosigneeDetails');
            $order->party_details = $request->get('txtPartyDetails');
            $order->date = $request->get('dteDate');
            $order->delivery_date = $request->get('dteDeliveryDate');
            $order->quantity_pieces = $request->get('txtQtyPieces');
            $order->product_code = $request->get('txtProductCode');
            $order->product_dimensions = $request->get('txtDimensions');
            $order->shipping_term_id = $request->get('selcShippingTerms');
            $order->product_mix_id = $request->get('selcProductMix');
            $order->washed_level_id = $request->get('selcWashedLevel');
            $order->naked_plank = $this->isBoolean($request->get('group1'));
            $order->slap_upside_down = $this->isBoolean($request->get('group2'));
            $order->drain_holes = $this->isBoolean($request->get('group3'));
            $order->dripper_holes = $this->isBoolean($request->get('group4'));
            $order->plant_hole_size_id =$request->get('selcPlantHole');
            $order->if_10_15_standing = $this->isBooleanns($request->get('standing'));
            $order->if_10_15_lying = $this->isBooleannl($request->get('lying'));
            $order->name_fill = $request->get('txtFillName');
            $order->status = true; 
            $save = $order->save();

            if ($save) {
                $responseBody = $this->responseBody(true, "Order", "saved", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    private function isBoolean($val){
        if($val == 'Yes'){
            return true;
        }
        return false;
    }


    private function isBooleanns($val){
        if($val == ('standing')){
            return true;
        }
        return false;
    }

    private function isBooleannl($val){
        if($val == ('lying')){
            return true;
        }
        return false;
    }

    
    /**
     * allOrder
     * This function is used to search all Order records from the database.
     * @return Json This returns as response.
     */
    public function allOrder()
    {

        try {
            $order =  Order::orderBy('customer_name')->get();
            $responseBody = $this->responseBody(true, "Order", "all", $order);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    /**
     * settings
     * This function is used to search settings records from the database.
     * @return Json This returns as response.
     */
    public function settings()
    {

        try {
           
            $shippingTerm = SettingsController::allEnableSettings("ShippingTerm");
            $productMix = SettingsController::allEnableSettings("ProductMix");
            $washedLevel = SettingsController::allEnableSettings("WashedLevel");
            $plantHoleSize = SettingsController::allEnableSettings("PlantHoleSize");

            $result = [
                "ShippingTerm" => $shippingTerm,
                "ProductMix" => $productMix,
                "WashedLevel" =>  $washedLevel,
                "PlantHoleSize" => $plantHoleSize,

            ];
            $responseBody = $this->responseBody(true, "Settings", "all", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }


    /**
     * order
     * This function is used to search recruitment from order id.
     * @return Json This returns as response.
     */
    public function order($id)
    {

        try {
            $order = Order::find($id);
            $shippingterm = SettingsController::getSettingsFromID('ShippingTerm', $order->shipping_term_id);
            $productmix = SettingsController::getSettingsFromID('ProductMix', $order->product_mix_id);
            $washedlevel = SettingsController::getSettingsFromID('WashedLevel', $order->washed_level_id);
            $plantholesize = SettingsController::getSettingsFromID('PlantHoleSize', $order->plant_hole_size_id);
            


            $settings = [
                "shippingterm" => $shippingterm->shipping_term,
                "productmix" => $productmix->product_mix,
                "washedlevel" => $washedlevel->washed_level,
                "plantholesize" => $plantholesize->plant_hole_size,
            ];

            $responseBody = $this->responseBody(true, "Order", "all", ["order" => $order, "settings" => $settings]);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }




    /**
     * delete
     * This function is used to delete order from recruitment id.
     * @return Json This returns as response.
     */
    public function delete($id)
    {
        try {
            $order = Order::find($id);
            $delete = $order->delete();
            if ($delete) {
                $responseBody = $this->responseBody(true, "Order", "deleted", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * update
     * This function is used to update order.
     * @param request This is the paramter to update function
     * @return Json This returns as response.
     */
    public function update(Request $request)
    {

       
        try {

            $id = $request->get('hidOrderID');
            $order = Order::find($id);
            $order->customer_name = $request->get('name');
            $order->purchase_order = $request->get('txtPurchase');
            //$order->product_name = $request->get('txtProductName');
            $order->bill_address = $request->get('txtBillAddress');
            $order->delivery_address = $request->get('txtDeliveryAddress');
            $order->cosignee_details = $request->get('txtCosigneeDetails');
            $order->party_details = $request->get('txtPartyDetails');
            $order->date = $request->get('dteDate');
            $order->delivery_date = $request->get('dteDeliveryDate');
            $order->quantity_pieces = $request->get('txtQtyPieces');
            $order->product_code = $request->get('txtProductCode');
            $order->product_dimensions = $request->get('txtDimensions');
            $order->shipping_term_id = $request->get('selcShippingTerms');
            $order->product_mix_id = $request->get('selcProductMix');
            $order->washed_level_id = $request->get('selcWashedLevel');
            $order->naked_plank = $this->isBoolean($request->get('group1'));
            $order->slap_upside_down = $this->isBoolean($request->get('group2'));
            $order->drain_holes = $this->isBoolean($request->get('group3'));
            $order->dripper_holes = $this->isBoolean($request->get('group4'));
            $order->plant_hole_size_id =$request->get('selcPlantHole');
            $order->if_10_15_standing = $this->isBooleanns($request->get('standing'));
            $order->if_10_15_lying = $this->isBooleannl($request->get('lying'));
            $order->name_fill = $request->get('txtFillName');
            $save = $order->save();
           
            $save = $order->save();
        

            if ($save) {
                $responseBody = $this->responseBody(true, "Order", "updated", "");
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    /**
     * disableTown
     * This function is used to disable town records.
     * @param request This is the paramter to disableTown function
     * @param id This is the paramter for identify update record.
     */
    public function disable(Request $request, $id)
    {
        try {
           
            $value = $request->get('status');

            $object = Order::find($id);
            $object->status = $value;
            $update = $object->save();
            if ($update) {
                $responseBody = $this->responseBody(true, "Order", "updated", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "Order", "error", null);
        }
        return response()->json(["data" => $responseBody]);
    }

}
