<?php

namespace Modules\Pp\Http\Controllers;

use Exception;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Am\Entities\Site;
use Modules\Hr\Entities\AtdHoliday;
use Modules\Pp\Entities\CustomerPlanData;
use Modules\Pp\Entities\EmployeeSite;
use Modules\Pp\Entities\Machinary;
use Modules\Pp\Entities\MachineryPlan;

class ProductionPlanMachineWiseController extends Controller
{
    function loadsite()
    {
        try {
            $Site = EmployeeSite::all();

            $responseBody = $this->responseBody(true, "Site Ids", "Found", $Site);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "Site Ids", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function loadMachinesTosite($id)
    {
        try {
            $machines = Machinary::where('site_id', $id)->get();
            return $this->responseBody(true, "loadMachinesTosite", "found", $machines);
        } catch (Exception $exception) {
            return $this->responseBody(false, "loadMachinesTosite", "error", $exception->getMessage());
        }
    }




    public function dateRange($startDate, $endDate)
    {

        $startjd = juliantojd(explode("-", $startDate)[1], explode("-", $startDate)[2], explode("-", $startDate)[0]);
        $endjd = juliantojd(explode("-", $endDate)[1], explode("-", $endDate)[2], explode("-", $endDate)[0]);

        $length = ($endjd - $startjd);
        $date_arr = [];
        for ($i = 0; $i < $length; $i++) {
            $date = jdtojulian(($startjd + $i));
            $month = explode("/", $date)[0];
            $day = explode("/", $date)[1];
            $year = explode("/", $date)[2];

            if (strlen($month) == 1) {
                $month = "0" . $month;
            }
            if (strlen($day) == 1) {
                $day = "0" . $day;
            }
            array_push($date_arr, array("dd" => ($year . "-" . $month . "-" . $day), "type" => 'week_day'));
        }


        for ($i = 0; $i < count($date_arr); $i++) {
            $holyday = AtdHoliday::where([['holyday_date', '=', $date_arr[$i]['dd']], ['holyday_type_id', '=', '2']])->first();
            if ($holyday) {
                $date_arr[$i]['type'] =  'holyday';
            }
        }

        return $this->responseBody(true, "ProductionPlanMachineWise", "dateRange", $date_arr);
    }


    public function createSchedule($startDate, $endDate, $quantity, $dayNight)
    {
        $startjd = juliantojd(explode("-", $startDate)[1], explode("-", $startDate)[2], explode("-", $startDate)[0]);
        $endjd = juliantojd(explode("-", $endDate)[1], explode("-", $endDate)[2], explode("-", $endDate)[0]);

        $length = ($endjd - $startjd);
        $date_arr = [];
        for ($i = 0; $i <= $length; $i++) {
            $date = jdtojulian(($startjd + $i));
            $month = explode("/", $date)[0];
            $day = explode("/", $date)[1];
            $year = explode("/", $date)[2];

            if (strlen($month) == 1) {
                $month = "0" . $month;
            }
            if (strlen($day) == 1) {
                $day = "0" . $day;
            }
            array_push($date_arr, ($year . "-" . $month . "-" . $day));
        }


        $day_count = 0;
        for ($i = 0; $i < count($date_arr); $i++) {
            $day_count++;
            $holyday = AtdHoliday::where([['holyday_date', '=', $date_arr[$i]], ['holyday_type_id', '=', '2']])->first();
            if ($holyday) {
                $day_count--;
                // unset($date_arr[$i]);
            }
        }

        $qty = ($quantity / $day_count);


        $data_arr = [];
        foreach ($date_arr as $date) {
            if ($dayNight == "true") {
                array_push($data_arr, ["date" => $date, "day_quantity" => ($qty / 2.0), "night_quantity" => ($qty / 2.0)]);
            } else {
                array_push($data_arr, ["date" => $date, "day_quantity" => $qty, "night_quantity" => 0]);
            }
        }

        return $this->responseBody(true, "ProductionPlanMachineWise", "dateRange", $data_arr);
    }


    public function saveMachineryPlan(Request $request)
    {
        try {
            $data = $request->get('data');
            for ($i = 0; $i < count($data); $i++) {
                $mplan = new MachineryPlan();
                $mplan->customer_order_plan_data_id = $data[$i]['customer_order_plan_data_id'];
                $mplan->machine_id = $data[$i]['machine_id'];
                $mplan->product_id = $data[$i]['product_id'];
                $mplan->date = $data[$i]['date'];
                $mplan->day_quantity = $data[$i]['day_quantity'];
                $mplan->night_quantity = $data[$i]['night_quantity'];
                $mplan->save();
            }

            $responseBody = $this->responseBody(true, "ProductionPlanMachineWise", "saveMachineryPlan", true);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ProductionPlanMachineWise", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function loadMachineryPlan($machine_id)
    {

        try {
            $query = 'SELECT pp_machinery_plans.customer_order_plan_data_id, date,"Planed" As product_id,"Planed" As product_name, SUM(pp_machinery_plans.day_quantity) As day_quantity, SUM(pp_machinery_plans.night_quantity) As night_quantity
            FROM pp_machinery_plans WHERE pp_machinery_plans.machine_id = "' . $machine_id . '"  GROUP BY date';

            $result = DB::select($query);
            $responseBody = $this->responseBody(true, "ProductionPlanMachineWise", "loadMachineryPlan", $result);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "ProductionPlanMachineWise", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function editMachineryPlan($order_plan_id, $startDate, $endDate)
    {
        try {
            $query = 'SELECT pp_customer_order_plans_data.*,pp_machinery_plans.*FROM pp_machinery_plans 
            INNER JOIN pp_customer_order_plans_data 
            ON pp_machinery_plans.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id 
            WHERE pp_machinery_plans.customer_order_plan_data_id =  "' . $order_plan_id . '"
            AND pp_machinery_plans.date BETWEEN  "' . $startDate . '"  AND  "' . $endDate . '" ';

            $result = DB::select($query);
            $responseBody = $this->responseBody(true, "ProductionPlanMachineWise", "editMachineryPlan", $result);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "ProductionPlanMachineWise", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function updateMachineryPlan(Request $request, $order_plan_id, $startDate, $endDate)
    {
        try {
            $query = 'DELETE FROM pp_machinery_plans 
            WHERE pp_machinery_plans.customer_order_plan_data_id =  "' . $order_plan_id . '"
            AND pp_machinery_plans.date BETWEEN  "' . $startDate . '"  AND  "' . $endDate . '" ';

            DB::select($query);
            $data = $request->get('data');
            for ($i = 0; $i < count($data); $i++) {
                $mplan = new MachineryPlan();
                $mplan->customer_order_plan_data_id = $data[$i]['customer_order_plan_data_id'];
                $mplan->machine_id = $data[$i]['machine_id'];
                $mplan->product_id = $data[$i]['product_id'];
                $mplan->date = $data[$i]['date'];
                $mplan->day_quantity = $data[$i]['day_quantity'];
                $mplan->night_quantity = $data[$i]['night_quantity'];
                $mplan->save();
            }
            $responseBody = $this->responseBody(true, "ProductionPlanMachineWise", "updateMachineryPlan", true);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "ProductionPlanMachineWise", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }




    public function allOrderPlanData($id)
    {

        try {
            //$customerPlanData = CustomerPlanData::find($id);
            $customerPlanData =  CustomerPlanData::where('customer_order_plan_id', $id)->get();
            foreach ($customerPlanData as $plandata) {
                $query = 'SELECT (SUM(pp_machinery_plans.day_quantity)+SUM(pp_machinery_plans.night_quantity)) AS planed FROM pp_machinery_plans WHERE pp_machinery_plans.customer_order_plan_data_id = "' . $plandata->customer_order_plan_data_id . '" AND pp_machinery_plans.date BETWEEN "' . $plandata->start_date . '" AND "' . $plandata->end_date . '"';
                $planed = DB::select($query)[0]->planed;
                if (!$planed) {
                    $plandata->planed = 0;
                } else {
                    $plandata->planed = $planed;
                }
            }
            $responseBody = $this->responseBody(true, "CustomerPlanData", "all", $customerPlanData);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function showPlanedModalData($date, $id)
    {

        try {
            $query = 'SELECT pp_customer_orders.purchase_order,pp_customer_order_plans_data.customer_order_plan_data_id,pp_customer_order_plans_data.manufacturing_order_no,pp_customer_order_plans_data.customer_order_plan_id,
            pp_customer_order_plans_data.product_id,pp_customer_order_plans_data.product_name,pp_machinery_plans.day_quantity,
            pp_machinery_plans.night_quantity FROM pp_machinery_plans INNER JOIN pp_customer_order_plans_data ON pp_machinery_plans.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN pp_customer_orders ON pp_customer_order_plans_data.customer_order_plan_data_id =  pp_customer_orders.order_id
            WHERE pp_machinery_plans.date = "' . $date . '"';

            $result = DB::select($query);
            $responseBody = $this->responseBody(true, "ProductionPlanMachineWise", "showPlanedModalData", $result);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "ProductionPlanMachineWise", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function loadAllPlanedDataMachine($product_id,$cus_order_id,$machine_id){

        try {
            $query = 'SELECT pp_customer_order_plans_data.*,pp_machinery_plans.*FROM pp_machinery_plans 
            INNER JOIN pp_customer_order_plans_data 
            ON pp_machinery_plans.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id 
            WHERE pp_machinery_plans.product_id = "' . $product_id . '" AND pp_machinery_plans.machine_id = "' . $machine_id . '"';

            $result = DB::select($query);
            $responseBody = $this->responseBody(true, "ProductionPlanMachineWise", "loadAllPlanedDataMachine", $result);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "ProductionPlanMachineWise", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }



    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
